@@ -1,5 +1,7 @@
 
-// $Id: rotated.c,v 1.6 2003/05/30 18:54:45 we7u Exp $
+// $Id: rotated.c,v 1.14 2005/01/08 10:06:54 we7u Exp $
+//
+// Portions Copyright (C) 2000-2005  The Xastir Group
 //
 // Note that this version has been changed since xvertext 5.0 in order
 // to get rid of compiler warnings and such.  The original 5.0 notice
@@ -39,6 +41,7 @@
 #include <math.h>
 #include <string.h>
 #include "rotated.h"
+#include "snprintf.h"
 
 
 /* ---------------------------------------------------------------------- */
@@ -164,7 +167,9 @@ static XImage *XRotMagnifyImage(Display *, XImage *);
 /**************************************************************************/
 /* Routine to mimic `strdup()' (some machines don't have it)              */
 /**************************************************************************/
-
+//
+// This function allocates some memory without freeing it.
+//
 static char *my_strdup(char *str) {
     char *s;
     
@@ -172,8 +177,16 @@ static char *my_strdup(char *str) {
         return NULL;
     
     s=(char *)malloc((unsigned)(strlen(str)+1));
-    if(s!=NULL) 
-        strcpy(s, str);
+
+    if(s!=NULL) {
+        xastir_snprintf(s,
+            strlen(str)+1,
+            "%s",
+            str);
+    }
+    else {
+        fprintf(stderr,"Couldn't allocate memory in my_strdup()\n");
+    }
     
     return s;
 }
@@ -238,7 +251,7 @@ static char *my_strtok(char *str1, char *str2) {
 
 float XRotVersion(char *str, int n) {
     if(str!=NULL)
-        strncpy(str, XV_COPYRIGHT, n);
+        xastir_snprintf(str, n, "%s", XV_COPYRIGHT);
     return XV_VERSION;
 }
 
@@ -275,15 +288,19 @@ void XRotSetBoundingBoxPad(int p) {
 /**************************************************************************/
 /*  Create an XImage structure and allocate memory for it                 */
 /**************************************************************************/
-
+//
+// Function does not free the memory.
+//
 static XImage *MakeXImage(Display *dpy, int w, int h) {
     XImage *I;
     char *data;
     
     /* reserve memory for image */
     data=(char *)calloc((unsigned)(((w-1)/8+1)*h), 1);
-    if(data==NULL)
+    if(data==NULL) {
+        fprintf(stderr,"Couldn't allocate memory in MakeXImage()\n");
         return NULL;
+    }
     
     /* create the XImage */
     I=XCreateImage(dpy, DefaultVisual(dpy, DefaultScreen(dpy)), 1, XYBitmap,
@@ -358,7 +375,10 @@ int XRotDrawAlignedImageString( Display *dpy, XFontStruct *font, float angle, Dr
 /**************************************************************************/
 /*  Aligns and paints a rotated string                                    */
 /**************************************************************************/
-
+//
+// This function allocates some memory but appears to free it
+// properly.
+//
 static int XRotPaintAlignedString( Display *dpy, XFontStruct *font, float angle, Drawable drawable, GC gc, int x, int y, char *text, int align, int bg) {
     int i;
     GC my_gc;
@@ -435,10 +455,12 @@ static int XRotPaintAlignedString( Display *dpy, XFontStruct *font, float angle,
         XPoint *xpoints;
         Pixmap empty_stipple;
         
-        /* reserve space for XPoints */
+        /* reserve space for XPoints, free'd later */
         xpoints=(XPoint *)malloc((unsigned)(4*item->nl*sizeof(XPoint)));
-        if(!xpoints)
+        if(!xpoints) {
+            fprintf(stderr,"Couldn't allocate memory in XRotPaintAlignedString()\n");
             return 1;
+        }
         
         /* rotate corner positions */
         for(i=0; i<4*item->nl; i++) {
@@ -596,7 +618,10 @@ static int XRotPaintAlignedString( Display *dpy, XFontStruct *font, float angle,
 /**************************************************************************/
 /*  Draw a horizontal string in a quick fashion                           */
 /**************************************************************************/
-
+//
+// This function allocates some memory but appears to free it
+// properly.
+//
 static int XRotDrawHorizontalString( Display *dpy, XFontStruct *font, Drawable drawable, GC gc, int x, int y, char *text, int align, int bg) {
     GC my_gc;
     int nl=1, i;
@@ -640,7 +665,8 @@ static int XRotDrawHorizontalString( Display *dpy, XFontStruct *font, Drawable d
         yp=y-nl*height+font->ascent;
     else
         yp=y;
-    
+   
+// Allocates some memory 
     str1=my_strdup(text);
     if(str1==NULL)
         return 1;
@@ -774,10 +800,12 @@ static RotatedTextItem *XRotRetrieveFromCache( Display *dpy, XFontStruct *font,
             return NULL;
 
         /* record what it shows */
+// Allocates some memory
         item->text=my_strdup(text);
 
         /* fontname or ID */
         if(font_name!=NULL) {
+// Allocates some memory
             item->font_name=my_strdup(font_name);
             item->fid=0;
         }
@@ -830,7 +858,10 @@ static RotatedTextItem *XRotRetrieveFromCache( Display *dpy, XFontStruct *font,
 /**************************************************************************/
 /*  Create a rotated text item                                            */
 /**************************************************************************/
-
+//
+// This function allocates some memory but appears to free it
+// properly.
+//
 static RotatedTextItem *XRotCreateTextItem( Display *dpy, XFontStruct *font, float angle, char *text, int align) {
     RotatedTextItem *item=NULL;
     Pixmap canvas;
@@ -851,11 +882,14 @@ static RotatedTextItem *XRotCreateTextItem( Display *dpy, XFontStruct *font, flo
     int dir, asc, desc;
     XCharStruct overall;
     int old_cols_in=0, old_rows_in=0;
+    int cols_in2, rows_in2;
     
     /* allocate memory */
     item=(RotatedTextItem *)malloc((unsigned)sizeof(RotatedTextItem));
-    if(!item)
+    if(!item) {
+        fprintf(stderr,"Couldn't allocate memory in RotatedTextItem()\n");
         return NULL;
+    }
         
     /* count number of sections in string */
     item->nl=1;
@@ -871,6 +905,7 @@ static RotatedTextItem *XRotCreateTextItem( Display *dpy, XFontStruct *font, flo
         str2=str2_b;
     
     /* find width of longest section */
+// Allocates some memory
     str1=my_strdup(text);
     if(str1==NULL)
         return NULL;
@@ -927,19 +962,24 @@ static RotatedTextItem *XRotCreateTextItem( Display *dpy, XFontStruct *font, flo
     /* text background will be drawn using XFillPolygon */
     item->corners_x=
         (float *)malloc((unsigned)(4*item->nl*sizeof(float)));
-    if(!item->corners_x)
+    if(!item->corners_x) {
+        fprintf(stderr,"Couldn't allocate memory in RotatedTextItem()\n");
         return NULL;
+    }
     
     item->corners_y=
         (float *)malloc((unsigned)(4*item->nl*sizeof(float)));
-    if(!item->corners_y)
+    if(!item->corners_y) {
+        fprintf(stderr,"Couldn't allocate memory in RotatedTextItem()\n");
         return NULL;
+    }
     
     /* draw text horizontally */
     
     /* start at top of bitmap */
     yp=font->ascent;
     
+// Allocates some memory
     str1=my_strdup(text);
     if(str1==NULL)
         return NULL;
@@ -1060,20 +1100,44 @@ static RotatedTextItem *XRotCreateTextItem( Display *dpy, XFontStruct *font, flo
         xinc=1./tan(angle);
     }
 
+
+    // Set up some constants for loops below
+    cols_in2 = item->cols_in / 2;
+    rows_in2 = item->rows_in / 2;
+ 
+
     /* loop through all relevent bits in rotated image */
     for(j=0; j<item->rows_out; j++) {
+        float dj_sin, dj_cos;
+        float temp_it, temp_jt;
+
         
         /* no point re-calculating these every pass */
         di=(float)((xl<0)?0:(int)xl)+0.5-(float)item->cols_out/2;
         byte_out=(item->rows_out-j-1)*byte_w_out;
-        
+
+        // New code:  Calculate these outside the inner loop for
+        // major speed gains.
+        dj_sin = dj * sin_angle;
+        dj_cos = dj * cos_angle;
+        temp_it = (float)cols_in2 + dj_sin;
+        temp_jt = (float)rows_in2 - dj_cos;
+
         /* loop through meaningful columns */
         for(i=((xl<0)?0:(int)xl); 
-            i<((xr>=item->cols_out)?item->cols_out:(int)xr); i++) {
+                i<((xr>=item->cols_out)?item->cols_out:(int)xr); i++) {
             
-            /* rotate coordinates */
-            it=(float)item->cols_in/2 + ( di*cos_angle + dj*sin_angle);
-            jt=(float)item->rows_in/2 - (-di*sin_angle + dj*cos_angle);
+            // rotate coordinates
+
+            // Original code (very slow)
+            //it=(float)item->cols_in/2 + ( di*cos_angle + dj*sin_angle);
+            //jt=(float)item->rows_in/2 - (-di*sin_angle + dj*cos_angle);
+
+            // New code (much faster)
+            //it=(float)cols_in2 + ( di*cos_angle + dj_sin);
+            //jt=(float)rows_in2 - (-di*sin_angle + dj_cos);
+            it=(float)temp_it + di*cos_angle;
+            jt=(float)temp_jt + di*sin_angle;
             
             /* set pixel if required */
             if(it>=0 && it<item->cols_in && jt>=0 && jt<item->rows_in)
@@ -1365,7 +1429,10 @@ static XImage *XRotMagnifyImage( Display *dpy, XImage *ximage) {
 /**************************************************************************/
 /* Calculate the bounding box some text will have when painted            */
 /**************************************************************************/
-
+//
+// This function allocates some memory, frees all but xp_out which
+// is returned.
+//
 XPoint *XRotTextExtents( Display *dpy, XFontStruct *font, float angle, int x, int y, char *text, int align) {
     register int i;
     char *str1, *str2, *str3;
@@ -1405,6 +1472,7 @@ XPoint *XRotTextExtents( Display *dpy, XFontStruct *font, float angle, int x, in
             str2=str2_b;
 
     /* find width of longest section */
+// Allocates some memory, free'd below
     str1=my_strdup(text);
     if(str1==NULL)
             return NULL;
@@ -1461,12 +1529,16 @@ XPoint *XRotTextExtents( Display *dpy, XFontStruct *font, float angle, int x, in
    
     /* reserve space for XPoints */
     xp_in=(XPoint *)malloc((unsigned)(5*sizeof(XPoint)));
-    if(!xp_in)
+    if(!xp_in) {
+        fprintf(stderr,"Couldn't allocate memory in XRotTextExtents()\n");
         return NULL;
+    }
 
     xp_out=(XPoint *)malloc((unsigned)(5*sizeof(XPoint)));
-    if(!xp_out)
+    if(!xp_out) {
+        fprintf(stderr,"Couldn't allocate memory in XRotTextExtents()\n");
         return NULL;
+    }
 
     /* bounding box when horizontal, relative to bitmap centre */
     xp_in[0].x=-(float)cols_in*style.magnify/2-style.bbx_pad;
